# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _semanage
else:
    import _semanage

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


SEMANAGE_MSG_ERR = _semanage.SEMANAGE_MSG_ERR
SEMANAGE_MSG_WARN = _semanage.SEMANAGE_MSG_WARN
SEMANAGE_MSG_INFO = _semanage.SEMANAGE_MSG_INFO

def semanage_msg_get_level(handle):
    return _semanage.semanage_msg_get_level(handle)

def semanage_msg_get_channel(handle):
    return _semanage.semanage_msg_get_channel(handle)

def semanage_msg_get_fname(handle):
    return _semanage.semanage_msg_get_fname(handle)

def semanage_msg_set_callback(handle, msg_callback, msg_callback_arg):
    return _semanage.semanage_msg_set_callback(handle, msg_callback, msg_callback_arg)

def semanage_handle_create():
    return _semanage.semanage_handle_create()

def semanage_handle_destroy(arg1):
    return _semanage.semanage_handle_destroy(arg1)
SEMANAGE_CON_INVALID = _semanage.SEMANAGE_CON_INVALID
SEMANAGE_CON_DIRECT = _semanage.SEMANAGE_CON_DIRECT
SEMANAGE_CON_POLSERV_LOCAL = _semanage.SEMANAGE_CON_POLSERV_LOCAL
SEMANAGE_CON_POLSERV_REMOTE = _semanage.SEMANAGE_CON_POLSERV_REMOTE

def semanage_select_store(handle, path, storetype):
    return _semanage.semanage_select_store(handle, path, storetype)

def semanage_reload_policy(handle):
    return _semanage.semanage_reload_policy(handle)

def semanage_set_reload(handle, do_reload):
    return _semanage.semanage_set_reload(handle, do_reload)

def semanage_set_rebuild(handle, do_rebuild):
    return _semanage.semanage_set_rebuild(handle, do_rebuild)

def semanage_set_check_ext_changes(handle, do_check):
    return _semanage.semanage_set_check_ext_changes(handle, do_check)

def semanage_get_hll_compiler_path(sh, lang_ext):
    return _semanage.semanage_get_hll_compiler_path(sh, lang_ext)

def semanage_set_create_store(handle, create_store):
    return _semanage.semanage_set_create_store(handle, create_store)

def semanage_get_disable_dontaudit(handle):
    return _semanage.semanage_get_disable_dontaudit(handle)

def semanage_set_disable_dontaudit(handle, disable_dontaudit):
    return _semanage.semanage_set_disable_dontaudit(handle, disable_dontaudit)

def semanage_set_check_contexts(sh, do_check_contexts):
    return _semanage.semanage_set_check_contexts(sh, do_check_contexts)

def semanage_get_default_priority(sh):
    return _semanage.semanage_get_default_priority(sh)

def semanage_set_default_priority(sh, priority):
    return _semanage.semanage_set_default_priority(sh, priority)

def semanage_is_managed(arg1):
    return _semanage.semanage_is_managed(arg1)

def semanage_connect(arg1):
    return _semanage.semanage_connect(arg1)

def semanage_disconnect(arg1):
    return _semanage.semanage_disconnect(arg1)

def semanage_begin_transaction(arg1):
    return _semanage.semanage_begin_transaction(arg1)

def semanage_commit(arg1):
    return _semanage.semanage_commit(arg1)
SEMANAGE_CAN_READ = _semanage.SEMANAGE_CAN_READ
SEMANAGE_CAN_WRITE = _semanage.SEMANAGE_CAN_WRITE

def semanage_access_check(sh):
    return _semanage.semanage_access_check(sh)

def semanage_is_connected(sh):
    return _semanage.semanage_is_connected(sh)

def semanage_mls_enabled(sh):
    return _semanage.semanage_mls_enabled(sh)

def semanage_set_root(path):
    return _semanage.semanage_set_root(path)

def semanage_root():
    return _semanage.semanage_root()

def semanage_get_preserve_tunables(handle):
    return _semanage.semanage_get_preserve_tunables(handle)

def semanage_set_preserve_tunables(handle, preserve_tunables):
    return _semanage.semanage_set_preserve_tunables(handle, preserve_tunables)

def semanage_get_ignore_module_cache(handle):
    return _semanage.semanage_get_ignore_module_cache(handle)

def semanage_set_ignore_module_cache(handle, ignore_module_cache):
    return _semanage.semanage_set_ignore_module_cache(handle, ignore_module_cache)

def semanage_set_store_root(sh, store_root):
    return _semanage.semanage_set_store_root(sh, store_root)

def semanage_module_install(arg1, module_data, data_len, name, ext_lang):
    return _semanage.semanage_module_install(arg1, module_data, data_len, name, ext_lang)

def semanage_module_install_file(arg1, module_name):
    return _semanage.semanage_module_install_file(arg1, module_name)

def semanage_module_remove(arg1, module_name):
    return _semanage.semanage_module_remove(arg1, module_name)

def semanage_module_extract(sh, modkey, extract_cil):
    return _semanage.semanage_module_extract(sh, modkey, extract_cil)

def semanage_module_list(arg1):
    return _semanage.semanage_module_list(arg1)

def semanage_module_info_datum_destroy(arg1):
    return _semanage.semanage_module_info_datum_destroy(arg1)

def semanage_module_list_nth(list, n):
    return _semanage.semanage_module_list_nth(list, n)

def semanage_module_get_name(arg1):
    return _semanage.semanage_module_get_name(arg1)

def semanage_module_info_create(sh):
    return _semanage.semanage_module_info_create(sh)

def semanage_module_info_destroy(handle, modinfo):
    return _semanage.semanage_module_info_destroy(handle, modinfo)

def semanage_module_info_get_priority(sh, modinfo):
    return _semanage.semanage_module_info_get_priority(sh, modinfo)

def semanage_module_info_get_name(sh, modinfo):
    return _semanage.semanage_module_info_get_name(sh, modinfo)

def semanage_module_info_get_lang_ext(sh, modinfo):
    return _semanage.semanage_module_info_get_lang_ext(sh, modinfo)

def semanage_module_info_get_enabled(sh, modinfo):
    return _semanage.semanage_module_info_get_enabled(sh, modinfo)

def semanage_module_info_set_priority(sh, modinfo, priority):
    return _semanage.semanage_module_info_set_priority(sh, modinfo, priority)

def semanage_module_info_set_name(sh, modinfo, name):
    return _semanage.semanage_module_info_set_name(sh, modinfo, name)

def semanage_module_info_set_lang_ext(sh, modinfo, lang_ext):
    return _semanage.semanage_module_info_set_lang_ext(sh, modinfo, lang_ext)

def semanage_module_info_set_enabled(sh, modinfo, enabled):
    return _semanage.semanage_module_info_set_enabled(sh, modinfo, enabled)

def semanage_module_key_create(sh):
    return _semanage.semanage_module_key_create(sh)

def semanage_module_key_destroy(sh, modkey):
    return _semanage.semanage_module_key_destroy(sh, modkey)

def semanage_module_key_get_name(sh, modkey):
    return _semanage.semanage_module_key_get_name(sh, modkey)

def semanage_module_key_get_priority(sh, modkey):
    return _semanage.semanage_module_key_get_priority(sh, modkey)

def semanage_module_key_set_name(sh, modkey, name):
    return _semanage.semanage_module_key_set_name(sh, modkey, name)

def semanage_module_key_set_priority(sh, modkey, priority):
    return _semanage.semanage_module_key_set_priority(sh, modkey, priority)

def semanage_module_set_enabled(sh, modkey, enabled):
    return _semanage.semanage_module_set_enabled(sh, modkey, enabled)

def semanage_module_get_module_info(sh, modkey):
    return _semanage.semanage_module_get_module_info(sh, modkey)

def semanage_module_list_all(sh):
    return _semanage.semanage_module_list_all(sh)

def semanage_module_install_info(sh, modinfo, data, data_len):
    return _semanage.semanage_module_install_info(sh, modinfo, data, data_len)

def semanage_module_remove_key(sh, modkey):
    return _semanage.semanage_module_remove_key(sh, modkey)

def semanage_module_get_enabled(sh, modkey):
    return _semanage.semanage_module_get_enabled(sh, modkey)

def semanage_module_compute_checksum(sh, modkey, cil):
    return _semanage.semanage_module_compute_checksum(sh, modkey, cil)

def semanage_context_get_user(con):
    return _semanage.semanage_context_get_user(con)

def semanage_context_set_user(handle, con, user):
    return _semanage.semanage_context_set_user(handle, con, user)

def semanage_context_get_role(con):
    return _semanage.semanage_context_get_role(con)

def semanage_context_set_role(handle, con, role):
    return _semanage.semanage_context_set_role(handle, con, role)

def semanage_context_get_type(con):
    return _semanage.semanage_context_get_type(con)

def semanage_context_set_type(handle, con, type):
    return _semanage.semanage_context_set_type(handle, con, type)

def semanage_context_get_mls(con):
    return _semanage.semanage_context_get_mls(con)

def semanage_context_set_mls(handle, con, mls_range):
    return _semanage.semanage_context_set_mls(handle, con, mls_range)

def semanage_context_create(handle):
    return _semanage.semanage_context_create(handle)

def semanage_context_clone(handle, con):
    return _semanage.semanage_context_clone(handle, con)

def semanage_context_free(con):
    return _semanage.semanage_context_free(con)

def semanage_context_from_string(handle, str):
    return _semanage.semanage_context_from_string(handle, str)

def semanage_context_to_string(handle, con):
    return _semanage.semanage_context_to_string(handle, con)

def semanage_bool_key_create(handle, name):
    return _semanage.semanage_bool_key_create(handle, name)

def semanage_bool_key_extract(handle, boolean):
    return _semanage.semanage_bool_key_extract(handle, boolean)

def semanage_bool_key_free(key):
    return _semanage.semanage_bool_key_free(key)

def semanage_bool_compare(boolean, key):
    return _semanage.semanage_bool_compare(boolean, key)

def semanage_bool_compare2(boolean, boolean2):
    return _semanage.semanage_bool_compare2(boolean, boolean2)

def semanage_bool_get_name(boolean):
    return _semanage.semanage_bool_get_name(boolean)

def semanage_bool_set_name(handle, boolean, name):
    return _semanage.semanage_bool_set_name(handle, boolean, name)

def semanage_bool_get_value(boolean):
    return _semanage.semanage_bool_get_value(boolean)

def semanage_bool_set_value(boolean, value):
    return _semanage.semanage_bool_set_value(boolean, value)

def semanage_bool_create(handle):
    return _semanage.semanage_bool_create(handle)

def semanage_bool_clone(handle, boolean):
    return _semanage.semanage_bool_clone(handle, boolean)

def semanage_bool_free(boolean):
    return _semanage.semanage_bool_free(boolean)

def semanage_bool_query(handle, key):
    return _semanage.semanage_bool_query(handle, key)

def semanage_bool_exists(handle, key):
    return _semanage.semanage_bool_exists(handle, key)

def semanage_bool_count(handle):
    return _semanage.semanage_bool_count(handle)

def semanage_bool_iterate(handle, handler, handler_arg):
    return _semanage.semanage_bool_iterate(handle, handler, handler_arg)

def semanage_bool_list(handle):
    return _semanage.semanage_bool_list(handle)

def semanage_bool_modify_local(handle, key, data):
    return _semanage.semanage_bool_modify_local(handle, key, data)

def semanage_bool_del_local(handle, key):
    return _semanage.semanage_bool_del_local(handle, key)

def semanage_bool_query_local(handle, key):
    return _semanage.semanage_bool_query_local(handle, key)

def semanage_bool_exists_local(handle, key):
    return _semanage.semanage_bool_exists_local(handle, key)

def semanage_bool_count_local(handle):
    return _semanage.semanage_bool_count_local(handle)

def semanage_bool_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_bool_iterate_local(handle, handler, handler_arg)

def semanage_bool_list_local(handle):
    return _semanage.semanage_bool_list_local(handle)

def semanage_bool_set_active(handle, key, data):
    return _semanage.semanage_bool_set_active(handle, key, data)

def semanage_bool_query_active(handle, key):
    return _semanage.semanage_bool_query_active(handle, key)

def semanage_bool_exists_active(handle, key):
    return _semanage.semanage_bool_exists_active(handle, key)

def semanage_bool_count_active(handle):
    return _semanage.semanage_bool_count_active(handle)

def semanage_bool_iterate_active(handle, handler, handler_arg):
    return _semanage.semanage_bool_iterate_active(handle, handler, handler_arg)

def semanage_bool_list_active(handle):
    return _semanage.semanage_bool_list_active(handle)

def semanage_iface_compare(iface, key):
    return _semanage.semanage_iface_compare(iface, key)

def semanage_iface_compare2(iface, iface2):
    return _semanage.semanage_iface_compare2(iface, iface2)

def semanage_iface_key_create(handle, name):
    return _semanage.semanage_iface_key_create(handle, name)

def semanage_iface_key_extract(handle, iface):
    return _semanage.semanage_iface_key_extract(handle, iface)

def semanage_iface_key_free(key):
    return _semanage.semanage_iface_key_free(key)

def semanage_iface_get_name(iface):
    return _semanage.semanage_iface_get_name(iface)

def semanage_iface_set_name(handle, iface, name):
    return _semanage.semanage_iface_set_name(handle, iface, name)

def semanage_iface_get_ifcon(iface):
    return _semanage.semanage_iface_get_ifcon(iface)

def semanage_iface_set_ifcon(handle, iface, con):
    return _semanage.semanage_iface_set_ifcon(handle, iface, con)

def semanage_iface_get_msgcon(iface):
    return _semanage.semanage_iface_get_msgcon(iface)

def semanage_iface_set_msgcon(handle, iface, con):
    return _semanage.semanage_iface_set_msgcon(handle, iface, con)

def semanage_iface_create(handle):
    return _semanage.semanage_iface_create(handle)

def semanage_iface_clone(handle, iface):
    return _semanage.semanage_iface_clone(handle, iface)

def semanage_iface_free(iface):
    return _semanage.semanage_iface_free(iface)

def semanage_iface_modify_local(handle, key, data):
    return _semanage.semanage_iface_modify_local(handle, key, data)

def semanage_iface_del_local(handle, key):
    return _semanage.semanage_iface_del_local(handle, key)

def semanage_iface_query_local(handle, key):
    return _semanage.semanage_iface_query_local(handle, key)

def semanage_iface_exists_local(handle, key):
    return _semanage.semanage_iface_exists_local(handle, key)

def semanage_iface_count_local(handle):
    return _semanage.semanage_iface_count_local(handle)

def semanage_iface_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_iface_iterate_local(handle, handler, handler_arg)

def semanage_iface_list_local(handle):
    return _semanage.semanage_iface_list_local(handle)

def semanage_iface_query(handle, key):
    return _semanage.semanage_iface_query(handle, key)

def semanage_iface_exists(handle, key):
    return _semanage.semanage_iface_exists(handle, key)

def semanage_iface_count(handle):
    return _semanage.semanage_iface_count(handle)

def semanage_iface_iterate(handle, handler, handler_arg):
    return _semanage.semanage_iface_iterate(handle, handler, handler_arg)

def semanage_iface_list(handle):
    return _semanage.semanage_iface_list(handle)

def semanage_user_key_create(handle, name):
    return _semanage.semanage_user_key_create(handle, name)

def semanage_user_key_extract(handle, user):
    return _semanage.semanage_user_key_extract(handle, user)

def semanage_user_key_free(key):
    return _semanage.semanage_user_key_free(key)

def semanage_user_compare(user, key):
    return _semanage.semanage_user_compare(user, key)

def semanage_user_compare2(user, user2):
    return _semanage.semanage_user_compare2(user, user2)

def semanage_user_get_name(user):
    return _semanage.semanage_user_get_name(user)

def semanage_user_set_name(handle, user, name):
    return _semanage.semanage_user_set_name(handle, user, name)

def semanage_user_get_prefix(user):
    return _semanage.semanage_user_get_prefix(user)

def semanage_user_set_prefix(handle, user, name):
    return _semanage.semanage_user_set_prefix(handle, user, name)

def semanage_user_get_mlslevel(user):
    return _semanage.semanage_user_get_mlslevel(user)

def semanage_user_set_mlslevel(handle, user, mls_level):
    return _semanage.semanage_user_set_mlslevel(handle, user, mls_level)

def semanage_user_get_mlsrange(user):
    return _semanage.semanage_user_get_mlsrange(user)

def semanage_user_set_mlsrange(handle, user, mls_range):
    return _semanage.semanage_user_set_mlsrange(handle, user, mls_range)

def semanage_user_get_num_roles(user):
    return _semanage.semanage_user_get_num_roles(user)

def semanage_user_add_role(handle, user, role):
    return _semanage.semanage_user_add_role(handle, user, role)

def semanage_user_del_role(user, role):
    return _semanage.semanage_user_del_role(user, role)

def semanage_user_has_role(user, role):
    return _semanage.semanage_user_has_role(user, role)

def semanage_user_get_roles(handle, user):
    return _semanage.semanage_user_get_roles(handle, user)

def semanage_user_set_roles(handle, user, num_roles):
    return _semanage.semanage_user_set_roles(handle, user, num_roles)

def semanage_user_create(handle):
    return _semanage.semanage_user_create(handle)

def semanage_user_clone(handle, user):
    return _semanage.semanage_user_clone(handle, user)

def semanage_user_free(user):
    return _semanage.semanage_user_free(user)

def semanage_user_modify_local(handle, key, data):
    return _semanage.semanage_user_modify_local(handle, key, data)

def semanage_user_del_local(handle, key):
    return _semanage.semanage_user_del_local(handle, key)

def semanage_user_query_local(handle, key):
    return _semanage.semanage_user_query_local(handle, key)

def semanage_user_exists_local(handle, key):
    return _semanage.semanage_user_exists_local(handle, key)

def semanage_user_count_local(handle):
    return _semanage.semanage_user_count_local(handle)

def semanage_user_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_user_iterate_local(handle, handler, handler_arg)

def semanage_user_list_local(handle):
    return _semanage.semanage_user_list_local(handle)

def semanage_user_query(handle, key):
    return _semanage.semanage_user_query(handle, key)

def semanage_user_exists(handle, key):
    return _semanage.semanage_user_exists(handle, key)

def semanage_user_count(handle):
    return _semanage.semanage_user_count(handle)

def semanage_user_iterate(handle, handler, handler_arg):
    return _semanage.semanage_user_iterate(handle, handler, handler_arg)

def semanage_user_list(handle):
    return _semanage.semanage_user_list(handle)
SEMANAGE_PROTO_UDP = _semanage.SEMANAGE_PROTO_UDP
SEMANAGE_PROTO_TCP = _semanage.SEMANAGE_PROTO_TCP
SEMANAGE_PROTO_DCCP = _semanage.SEMANAGE_PROTO_DCCP
SEMANAGE_PROTO_SCTP = _semanage.SEMANAGE_PROTO_SCTP

def semanage_port_compare(port, key):
    return _semanage.semanage_port_compare(port, key)

def semanage_port_compare2(port, port2):
    return _semanage.semanage_port_compare2(port, port2)

def semanage_port_key_create(handle, low, high, proto):
    return _semanage.semanage_port_key_create(handle, low, high, proto)

def semanage_port_key_extract(handle, port):
    return _semanage.semanage_port_key_extract(handle, port)

def semanage_port_key_free(key):
    return _semanage.semanage_port_key_free(key)

def semanage_port_get_proto(port):
    return _semanage.semanage_port_get_proto(port)

def semanage_port_set_proto(port, proto):
    return _semanage.semanage_port_set_proto(port, proto)

def semanage_port_get_proto_str(proto):
    return _semanage.semanage_port_get_proto_str(proto)

def semanage_port_get_low(port):
    return _semanage.semanage_port_get_low(port)

def semanage_port_get_high(port):
    return _semanage.semanage_port_get_high(port)

def semanage_port_set_port(port, port_num):
    return _semanage.semanage_port_set_port(port, port_num)

def semanage_port_set_range(port, low, high):
    return _semanage.semanage_port_set_range(port, low, high)

def semanage_port_get_con(port):
    return _semanage.semanage_port_get_con(port)

def semanage_port_set_con(handle, port, con):
    return _semanage.semanage_port_set_con(handle, port, con)

def semanage_port_create(handle):
    return _semanage.semanage_port_create(handle)

def semanage_port_clone(handle, port):
    return _semanage.semanage_port_clone(handle, port)

def semanage_port_free(port):
    return _semanage.semanage_port_free(port)

def semanage_port_modify_local(handle, key, data):
    return _semanage.semanage_port_modify_local(handle, key, data)

def semanage_port_del_local(handle, key):
    return _semanage.semanage_port_del_local(handle, key)

def semanage_port_query_local(handle, key):
    return _semanage.semanage_port_query_local(handle, key)

def semanage_port_exists_local(handle, key):
    return _semanage.semanage_port_exists_local(handle, key)

def semanage_port_count_local(handle):
    return _semanage.semanage_port_count_local(handle)

def semanage_port_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_port_iterate_local(handle, handler, handler_arg)

def semanage_port_list_local(handle):
    return _semanage.semanage_port_list_local(handle)

def semanage_port_query(handle, key):
    return _semanage.semanage_port_query(handle, key)

def semanage_port_exists(handle, key):
    return _semanage.semanage_port_exists(handle, key)

def semanage_port_count(handle):
    return _semanage.semanage_port_count(handle)

def semanage_port_iterate(handle, handler, handler_arg):
    return _semanage.semanage_port_iterate(handle, handler, handler_arg)

def semanage_port_list(handle):
    return _semanage.semanage_port_list(handle)

def semanage_ibpkey_compare(ibpkey, key):
    return _semanage.semanage_ibpkey_compare(ibpkey, key)

def semanage_ibpkey_compare2(ibpkey, ibpkey2):
    return _semanage.semanage_ibpkey_compare2(ibpkey, ibpkey2)

def semanage_ibpkey_key_create(handle, subnet_prefix, low, high):
    return _semanage.semanage_ibpkey_key_create(handle, subnet_prefix, low, high)

def semanage_ibpkey_key_extract(handle, ibpkey):
    return _semanage.semanage_ibpkey_key_extract(handle, ibpkey)

def semanage_ibpkey_key_free(key):
    return _semanage.semanage_ibpkey_key_free(key)

def semanage_ibpkey_get_subnet_prefix(handle, ibpkey):
    return _semanage.semanage_ibpkey_get_subnet_prefix(handle, ibpkey)

def semanage_ibpkey_get_subnet_prefix_bytes(ibpkey):
    return _semanage.semanage_ibpkey_get_subnet_prefix_bytes(ibpkey)

def semanage_ibpkey_set_subnet_prefix(handle, ibpkey, subnet_prefix):
    return _semanage.semanage_ibpkey_set_subnet_prefix(handle, ibpkey, subnet_prefix)

def semanage_ibpkey_set_subnet_prefix_bytes(ibpkey, subnet_prefix):
    return _semanage.semanage_ibpkey_set_subnet_prefix_bytes(ibpkey, subnet_prefix)

def semanage_ibpkey_get_low(ibpkey):
    return _semanage.semanage_ibpkey_get_low(ibpkey)

def semanage_ibpkey_get_high(ibpkey):
    return _semanage.semanage_ibpkey_get_high(ibpkey)

def semanage_ibpkey_set_pkey(ibpkey, pkey_num):
    return _semanage.semanage_ibpkey_set_pkey(ibpkey, pkey_num)

def semanage_ibpkey_set_range(ibpkey, low, high):
    return _semanage.semanage_ibpkey_set_range(ibpkey, low, high)

def semanage_ibpkey_get_con(ibpkey):
    return _semanage.semanage_ibpkey_get_con(ibpkey)

def semanage_ibpkey_set_con(handle, ibpkey, con):
    return _semanage.semanage_ibpkey_set_con(handle, ibpkey, con)

def semanage_ibpkey_create(handle):
    return _semanage.semanage_ibpkey_create(handle)

def semanage_ibpkey_clone(handle, ibpkey):
    return _semanage.semanage_ibpkey_clone(handle, ibpkey)

def semanage_ibpkey_free(ibpkey):
    return _semanage.semanage_ibpkey_free(ibpkey)

def semanage_ibpkey_modify_local(handle, key, data):
    return _semanage.semanage_ibpkey_modify_local(handle, key, data)

def semanage_ibpkey_del_local(handle, key):
    return _semanage.semanage_ibpkey_del_local(handle, key)

def semanage_ibpkey_query_local(handle, key):
    return _semanage.semanage_ibpkey_query_local(handle, key)

def semanage_ibpkey_exists_local(handle, key):
    return _semanage.semanage_ibpkey_exists_local(handle, key)

def semanage_ibpkey_count_local(handle):
    return _semanage.semanage_ibpkey_count_local(handle)

def semanage_ibpkey_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_ibpkey_iterate_local(handle, handler, handler_arg)

def semanage_ibpkey_list_local(handle):
    return _semanage.semanage_ibpkey_list_local(handle)

def semanage_ibpkey_query(handle, key):
    return _semanage.semanage_ibpkey_query(handle, key)

def semanage_ibpkey_exists(handle, key):
    return _semanage.semanage_ibpkey_exists(handle, key)

def semanage_ibpkey_count(handle):
    return _semanage.semanage_ibpkey_count(handle)

def semanage_ibpkey_iterate(handle, handler, handler_arg):
    return _semanage.semanage_ibpkey_iterate(handle, handler, handler_arg)

def semanage_ibpkey_list(handle):
    return _semanage.semanage_ibpkey_list(handle)

def semanage_ibendport_compare(ibendport, key):
    return _semanage.semanage_ibendport_compare(ibendport, key)

def semanage_ibendport_compare2(ibendport, ibendport2):
    return _semanage.semanage_ibendport_compare2(ibendport, ibendport2)

def semanage_ibendport_key_create(handle, ibdev_name, port):
    return _semanage.semanage_ibendport_key_create(handle, ibdev_name, port)

def semanage_ibendport_key_extract(handle, ibendport):
    return _semanage.semanage_ibendport_key_extract(handle, ibendport)

def semanage_ibendport_key_free(key):
    return _semanage.semanage_ibendport_key_free(key)

def semanage_ibendport_get_ibdev_name(handle, ibendport):
    return _semanage.semanage_ibendport_get_ibdev_name(handle, ibendport)

def semanage_ibendport_set_ibdev_name(handle, ibendport, ibdev_name):
    return _semanage.semanage_ibendport_set_ibdev_name(handle, ibendport, ibdev_name)

def semanage_ibendport_get_port(ibendport):
    return _semanage.semanage_ibendport_get_port(ibendport)

def semanage_ibendport_set_port(ibendport, port):
    return _semanage.semanage_ibendport_set_port(ibendport, port)

def semanage_ibendport_get_con(ibendport):
    return _semanage.semanage_ibendport_get_con(ibendport)

def semanage_ibendport_set_con(handle, ibendport, con):
    return _semanage.semanage_ibendport_set_con(handle, ibendport, con)

def semanage_ibendport_create(handle):
    return _semanage.semanage_ibendport_create(handle)

def semanage_ibendport_clone(handle, ibendport):
    return _semanage.semanage_ibendport_clone(handle, ibendport)

def semanage_ibendport_free(ibendport):
    return _semanage.semanage_ibendport_free(ibendport)

def semanage_ibendport_modify_local(handle, key, data):
    return _semanage.semanage_ibendport_modify_local(handle, key, data)

def semanage_ibendport_del_local(handle, key):
    return _semanage.semanage_ibendport_del_local(handle, key)

def semanage_ibendport_query_local(handle, key):
    return _semanage.semanage_ibendport_query_local(handle, key)

def semanage_ibendport_exists_local(handle, key):
    return _semanage.semanage_ibendport_exists_local(handle, key)

def semanage_ibendport_count_local(handle):
    return _semanage.semanage_ibendport_count_local(handle)

def semanage_ibendport_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_ibendport_iterate_local(handle, handler, handler_arg)

def semanage_ibendport_list_local(handle):
    return _semanage.semanage_ibendport_list_local(handle)

def semanage_ibendport_query(handle, key):
    return _semanage.semanage_ibendport_query(handle, key)

def semanage_ibendport_exists(handle, key):
    return _semanage.semanage_ibendport_exists(handle, key)

def semanage_ibendport_count(handle):
    return _semanage.semanage_ibendport_count(handle)

def semanage_ibendport_iterate(handle, handler, handler_arg):
    return _semanage.semanage_ibendport_iterate(handle, handler, handler_arg)

def semanage_ibendport_list(handle):
    return _semanage.semanage_ibendport_list(handle)

def semanage_fcontext_compare(fcontext, key):
    return _semanage.semanage_fcontext_compare(fcontext, key)

def semanage_fcontext_compare2(fcontext, fcontext2):
    return _semanage.semanage_fcontext_compare2(fcontext, fcontext2)

def semanage_fcontext_key_create(handle, expr, type):
    return _semanage.semanage_fcontext_key_create(handle, expr, type)

def semanage_fcontext_key_extract(handle, fcontext):
    return _semanage.semanage_fcontext_key_extract(handle, fcontext)

def semanage_fcontext_key_free(key):
    return _semanage.semanage_fcontext_key_free(key)

def semanage_fcontext_get_expr(fcontext):
    return _semanage.semanage_fcontext_get_expr(fcontext)

def semanage_fcontext_set_expr(handle, fcontext, expr):
    return _semanage.semanage_fcontext_set_expr(handle, fcontext, expr)
SEMANAGE_FCONTEXT_ALL = _semanage.SEMANAGE_FCONTEXT_ALL
SEMANAGE_FCONTEXT_REG = _semanage.SEMANAGE_FCONTEXT_REG
SEMANAGE_FCONTEXT_DIR = _semanage.SEMANAGE_FCONTEXT_DIR
SEMANAGE_FCONTEXT_CHAR = _semanage.SEMANAGE_FCONTEXT_CHAR
SEMANAGE_FCONTEXT_BLOCK = _semanage.SEMANAGE_FCONTEXT_BLOCK
SEMANAGE_FCONTEXT_SOCK = _semanage.SEMANAGE_FCONTEXT_SOCK
SEMANAGE_FCONTEXT_LINK = _semanage.SEMANAGE_FCONTEXT_LINK
SEMANAGE_FCONTEXT_PIPE = _semanage.SEMANAGE_FCONTEXT_PIPE

def semanage_fcontext_get_type(fcontext):
    return _semanage.semanage_fcontext_get_type(fcontext)

def semanage_fcontext_get_type_str(type):
    return _semanage.semanage_fcontext_get_type_str(type)

def semanage_fcontext_set_type(fcontext, type):
    return _semanage.semanage_fcontext_set_type(fcontext, type)

def semanage_fcontext_get_con(fcontext):
    return _semanage.semanage_fcontext_get_con(fcontext)

def semanage_fcontext_set_con(handle, fcontext, con):
    return _semanage.semanage_fcontext_set_con(handle, fcontext, con)

def semanage_fcontext_create(handle):
    return _semanage.semanage_fcontext_create(handle)

def semanage_fcontext_clone(handle, fcontext):
    return _semanage.semanage_fcontext_clone(handle, fcontext)

def semanage_fcontext_free(fcontext):
    return _semanage.semanage_fcontext_free(fcontext)

def semanage_fcontext_modify_local(handle, key, data):
    return _semanage.semanage_fcontext_modify_local(handle, key, data)

def semanage_fcontext_del_local(handle, key):
    return _semanage.semanage_fcontext_del_local(handle, key)

def semanage_fcontext_query_local(handle, key):
    return _semanage.semanage_fcontext_query_local(handle, key)

def semanage_fcontext_exists_local(handle, key):
    return _semanage.semanage_fcontext_exists_local(handle, key)

def semanage_fcontext_count_local(handle):
    return _semanage.semanage_fcontext_count_local(handle)

def semanage_fcontext_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_fcontext_iterate_local(handle, handler, handler_arg)

def semanage_fcontext_list_local(handle):
    return _semanage.semanage_fcontext_list_local(handle)

def semanage_fcontext_query(handle, key):
    return _semanage.semanage_fcontext_query(handle, key)

def semanage_fcontext_exists(handle, key):
    return _semanage.semanage_fcontext_exists(handle, key)

def semanage_fcontext_count(handle):
    return _semanage.semanage_fcontext_count(handle)

def semanage_fcontext_iterate(handle, handler, handler_arg):
    return _semanage.semanage_fcontext_iterate(handle, handler, handler_arg)

def semanage_fcontext_list(handle):
    return _semanage.semanage_fcontext_list(handle)

def semanage_fcontext_list_homedirs(handle):
    return _semanage.semanage_fcontext_list_homedirs(handle)

def semanage_seuser_key_create(handle, name):
    return _semanage.semanage_seuser_key_create(handle, name)

def semanage_seuser_key_extract(handle, seuser):
    return _semanage.semanage_seuser_key_extract(handle, seuser)

def semanage_seuser_key_free(key):
    return _semanage.semanage_seuser_key_free(key)

def semanage_seuser_compare(seuser, key):
    return _semanage.semanage_seuser_compare(seuser, key)

def semanage_seuser_compare2(seuser, seuser2):
    return _semanage.semanage_seuser_compare2(seuser, seuser2)

def semanage_seuser_get_name(seuser):
    return _semanage.semanage_seuser_get_name(seuser)

def semanage_seuser_set_name(handle, seuser, name):
    return _semanage.semanage_seuser_set_name(handle, seuser, name)

def semanage_seuser_get_sename(seuser):
    return _semanage.semanage_seuser_get_sename(seuser)

def semanage_seuser_set_sename(handle, seuser, sename):
    return _semanage.semanage_seuser_set_sename(handle, seuser, sename)

def semanage_seuser_get_mlsrange(seuser):
    return _semanage.semanage_seuser_get_mlsrange(seuser)

def semanage_seuser_set_mlsrange(handle, seuser, mls_range):
    return _semanage.semanage_seuser_set_mlsrange(handle, seuser, mls_range)

def semanage_seuser_create(handle):
    return _semanage.semanage_seuser_create(handle)

def semanage_seuser_clone(handle, seuser):
    return _semanage.semanage_seuser_clone(handle, seuser)

def semanage_seuser_free(seuser):
    return _semanage.semanage_seuser_free(seuser)

def semanage_seuser_modify_local(handle, key, data):
    return _semanage.semanage_seuser_modify_local(handle, key, data)

def semanage_seuser_del_local(handle, key):
    return _semanage.semanage_seuser_del_local(handle, key)

def semanage_seuser_query_local(handle, key):
    return _semanage.semanage_seuser_query_local(handle, key)

def semanage_seuser_exists_local(handle, key):
    return _semanage.semanage_seuser_exists_local(handle, key)

def semanage_seuser_count_local(handle):
    return _semanage.semanage_seuser_count_local(handle)

def semanage_seuser_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_seuser_iterate_local(handle, handler, handler_arg)

def semanage_seuser_list_local(handle):
    return _semanage.semanage_seuser_list_local(handle)

def semanage_seuser_query(handle, key):
    return _semanage.semanage_seuser_query(handle, key)

def semanage_seuser_exists(handle, key):
    return _semanage.semanage_seuser_exists(handle, key)

def semanage_seuser_count(handle):
    return _semanage.semanage_seuser_count(handle)

def semanage_seuser_iterate(handle, handler, handler_arg):
    return _semanage.semanage_seuser_iterate(handle, handler, handler_arg)

def semanage_seuser_list(handle):
    return _semanage.semanage_seuser_list(handle)
SEMANAGE_PROTO_IP4 = _semanage.SEMANAGE_PROTO_IP4
SEMANAGE_PROTO_IP6 = _semanage.SEMANAGE_PROTO_IP6

def semanage_node_compare(node, key):
    return _semanage.semanage_node_compare(node, key)

def semanage_node_compare2(node, node2):
    return _semanage.semanage_node_compare2(node, node2)

def semanage_node_key_create(handle, addr, mask, proto):
    return _semanage.semanage_node_key_create(handle, addr, mask, proto)

def semanage_node_key_extract(handle, node):
    return _semanage.semanage_node_key_extract(handle, node)

def semanage_node_key_free(key):
    return _semanage.semanage_node_key_free(key)

def semanage_node_get_addr(handle, node):
    return _semanage.semanage_node_get_addr(handle, node)

def semanage_node_get_addr_bytes(handle, node):
    return _semanage.semanage_node_get_addr_bytes(handle, node)

def semanage_node_set_addr(handle, node, proto, addr):
    return _semanage.semanage_node_set_addr(handle, node, proto, addr)

def semanage_node_set_addr_bytes(handle, node, addr, addr_sz):
    return _semanage.semanage_node_set_addr_bytes(handle, node, addr, addr_sz)

def semanage_node_get_mask(handle, node):
    return _semanage.semanage_node_get_mask(handle, node)

def semanage_node_get_mask_bytes(handle, node):
    return _semanage.semanage_node_get_mask_bytes(handle, node)

def semanage_node_set_mask(handle, node, proto, mask):
    return _semanage.semanage_node_set_mask(handle, node, proto, mask)

def semanage_node_set_mask_bytes(handle, node, mask, mask_sz):
    return _semanage.semanage_node_set_mask_bytes(handle, node, mask, mask_sz)

def semanage_node_get_proto(node):
    return _semanage.semanage_node_get_proto(node)

def semanage_node_set_proto(node, proto):
    return _semanage.semanage_node_set_proto(node, proto)

def semanage_node_get_proto_str(proto):
    return _semanage.semanage_node_get_proto_str(proto)

def semanage_node_get_con(node):
    return _semanage.semanage_node_get_con(node)

def semanage_node_set_con(handle, node, con):
    return _semanage.semanage_node_set_con(handle, node, con)

def semanage_node_create(handle):
    return _semanage.semanage_node_create(handle)

def semanage_node_clone(handle, node):
    return _semanage.semanage_node_clone(handle, node)

def semanage_node_free(node):
    return _semanage.semanage_node_free(node)

def semanage_node_modify_local(handle, key, data):
    return _semanage.semanage_node_modify_local(handle, key, data)

def semanage_node_del_local(handle, key):
    return _semanage.semanage_node_del_local(handle, key)

def semanage_node_query_local(handle, key):
    return _semanage.semanage_node_query_local(handle, key)

def semanage_node_exists_local(handle, key):
    return _semanage.semanage_node_exists_local(handle, key)

def semanage_node_count_local(handle):
    return _semanage.semanage_node_count_local(handle)

def semanage_node_iterate_local(handle, handler, handler_arg):
    return _semanage.semanage_node_iterate_local(handle, handler, handler_arg)

def semanage_node_list_local(handle):
    return _semanage.semanage_node_list_local(handle)

def semanage_node_query(handle, key):
    return _semanage.semanage_node_query(handle, key)

def semanage_node_exists(handle, key):
    return _semanage.semanage_node_exists(handle, key)

def semanage_node_count(handle):
    return _semanage.semanage_node_count(handle)

def semanage_node_iterate(handle, handler, handler_arg):
    return _semanage.semanage_node_iterate(handle, handler, handler_arg)

def semanage_node_list(handle):
    return _semanage.semanage_node_list(handle)


